'use strict';

import React, { Component } from 'react';
import ReactDOM from 'react-dom';
import PropTypes from 'prop-types';
import Classnames from 'classnames';
import Fade from './transition/fade';
import Tag from './tag';

/**
 * Bootstrap Tooltip integration
 * @see [Bootstrap Tooltip]{@link https://getbootstrap.com/docs/4.0/components/tooltips/}
 *
 * @class TooltipControlled
 * @extends React.Component
 * @author Jocelyn Faihy <jocelyn@faihy.fr>
 *
 * @root Theme.Tag
 * @property {Object} [props] - Component properties
 * @property {Boolean} [props.active] - Set popover to active state
 * @property {Function} [props.enable] - Set popover to active state
 * @property {Function} [props.disable] - Set popover to inactive state
 * @property {Function} [props.toggle] - Toggle popover to active state
 */
class TooltipControlled extends Component {
    static propTypes = {
        ...Tag.propTypes,
        active: PropTypes.bool,
        enable: PropTypes.func,
        disable: PropTypes.func,
        toggle: PropTypes.func,
    }

    static defaultProps = {
        tag: 'span',
    }

    static childContextTypes = {
        tooltip: PropTypes.shape({
            active: PropTypes.bool.isRequired,
            enable: PropTypes.func,
            disable: PropTypes.func,
            setTarget: PropTypes.func.isRequired,
            getTarget: PropTypes.func.isRequired,
        })
    };

    getChildContext = () => Object({
        tooltip: {
            active: this.props.active,
            enable: this.props.enable,
            disable: this.props.disable,
            setTarget: node => this.target = node,
            getTarget: () => this.target,
        }
    });

    render() {
        const { active, className, direction, enable, disable, ...props } = this.props;
        const classes = Classnames(className, 'tooltip-layout');
        return <Tag {...props} className={classes} />
    }
}

/**
 * Bootstrap Tooltip simplier integration
 * @see [Bootstrap Tooltip]{@link https://getbootstrap.com/docs/4.0/components/tooltips/}
 *
 * @class Tooltip
 * @extends React.Component
 * @author Jocelyn Faihy <jocelyn@faihy.fr>
 *
 * @root Theme.Tag
 * @property {Object} [props] - Component properties
 * @property {Boolean} [props.controlled] - Set tooltip to controlled state
 */
export default class Tooltip extends Component {
    static propTypes = {
        ...TooltipControlled.propTypes,
        controlled: PropTypes.bool,
    }

    state = { hovered: false, inside: false };
    enable = () => this.setState({ hovered: true, inside: true });
    disable = () => this.setState({ hovered: false });

    componentWillUpdate(props, state) {
        if (!state.hovered && state.hovered != this.state.hovered) {
            if (this.timeout) clearTimeout(this.timeout);
            this.timeout = setTimeout(() => this.setState({ inside: this.state.hovered }), 300);
        }
    }

    render() {
        let { controlled, ...props } = this.props;
        if (controlled) return <TooltipControlled {...props} />
        return <TooltipControlled {...props} active={this.state.inside} 
            enable={this.enable} disable={this.disable} 
        />
    }

    componentWillUnmount() {
        if (this.timeout) clearTimeout(this.timeout);
    }
}

/**
 * Bootstrap Tooltip Inner integration
 * @see [Bootstrap Tooltip]{@link https://getbootstrap.com/docs/4.0/components/tooltips/}
 *
 * @class TooltipInner
 * @extends React.Component
 * @author Jocelyn Faihy <jocelyn@faihy.fr>
 *
 * @root Theme.Tag
 * @property {Object} [props] - Component properties
 * @property {Boolean|String|Number} [props.arrow] - Specify what kind of arrow appear
 * @property {String} [props.direction] - Specify direction where popover will appear (auto|top|bottom|left|right)
 */
export class TooltipInner extends Component {
    static propTypes = {
        ...Fade.propTypes,
        arrow: PropTypes.oneOfType([PropTypes.bool, PropTypes.string, PropTypes.number]),
        direction: PropTypes.oneOf(['auto', 'top','bottom','left','right']),
    }

    static defaultProps = {
        tag: 'article',
        direction: 'auto',
    }

    static contextTypes = {
        tooltip: PropTypes.shape({
            active: PropTypes.bool.isRequired,
            enable: PropTypes.func,
            disable: PropTypes.func,
            setTarget: PropTypes.func.isRequired,
            getTarget: PropTypes.func.isRequired,
        })
    }

    render() {
        const { active } = this.context.tooltip;
        const { arrow, className, children, direction, ...props } = this.props;
        const classes = Classnames(className, 'tooltip', `tooltip-${this.props.direction}`, { active });
        const arrowClasses = Classnames(className, 'arrow', arrow === true && 'arrow-middle');

        return <Fade {...props} className={classes} active={this.context.tooltip.active} role="tooltip" 
            onMouseEnter={this.onMouseEnter} onMouseLeave={this.onMouseLeave}>
            {arrow && <section className={arrowClasses} />}
            {children}
        </Fade>
    }

    componentDidMount() {
        this.componentDidRendered();
    }

    componentDidUpdate() {
        this.componentDidRendered();
    }

    componentDidRendered() {
        if (this.context.tooltip.active) {
            const popper = ReactDOM.findDOMNode(this);
            const target = this.context.tooltip.getTarget();
            this.setOffsetLeft(target, popper, this.props.direction);
            this.setOffsetTop(target, popper, this.props.direction);
        }
    }

    onMouseEnter = event => {
        if (this.props.onMouseEnter) this.props.onMouseEnter(event);
        this.context.tooltip.enable();
    } 

    onMouseLeave = event => {
        if (this.props.onMouseLeave) this.props.onMouseLeave(event);
        this.context.tooltip.disable();
    }

    setOffsetLeft = (target, popper, direction) => {
        switch (direction) {
            case 'left':
                popper.style.left = 'auto';
                popper.style.right = `${target.offsetWidth}px`;
                break;
            case 'right':
                popper.style.left = `${target.offsetWidth}px`;
                break;
            case 'bottom':
            case 'top':
                popper.style.left = `${target.offsetWidth / 2 - popper.offsetWidth / 2}px`;
                break;
        }
    }

    setOffsetTop = (target, popper, direction) => {
        switch (direction) {
            case 'top':
                popper.style.top = 'auto';
                popper.style.bottom = `${target.offsetHeight}px`;
                break;
            case 'bottom':
                popper.style.top = `${target.offsetHeight}px`;
                break;
            case 'left':
            case 'right':
                popper.style.top = `${target.offsetHeight / 2 - popper.offsetHeight / 2}px`;
                break;
        }
    }
}

/**
 * Bootstrap Tooltip Toggle integration
 * @see [Bootstrap Tooltip]{@link https://getbootstrap.com/docs/4.0/components/tooltips/}
 *
 * @class TooltipToggle
 * @extends React.Component
 * @author Jocelyn Faihy <jocelyn@faihy.fr>
 *
 * @root Theme.Tag
 * @property {Object} [props] - Component properties
 */
export class TooltipToggle extends Component {
    static propTypes = {
        ...Tag.propTypes,
    };

    static defaultProps = {
        tag: 'span'
    };

    static contextTypes = {
        tooltip: PropTypes.shape({
            active: PropTypes.bool.isRequired,
            enable: PropTypes.func,
            disable: PropTypes.func,
            setTarget: PropTypes.func.isRequired,
            getTarget: PropTypes.func.isRequired,
        })
    };

    render() {
        const { className, ...props } = this.props;
        const classes = Classnames(className, 'tooltip-toggle');
        return <Tag {...props} className={classes} 
            onMouseEnter={this.onMouseEnter} 
            onMouseLeave={this.onMouseLeave} 
        />
    }

    componentDidMount() {
        this.context.tooltip.setTarget(ReactDOM.findDOMNode(this));
    }

    onMouseEnter = event => {
        if (this.props.onMouseEnter) this.props.onMouseEnter(event);
        this.context.tooltip.enable(event);
    }

    onMouseLeave = event => {
        if (this.props.onMouseLeave) this.props.onMouseLeave(event);
        if (this.context.tooltip.inside) return;
        this.context.tooltip.disable(event);
    }
}

/**
 * Bootstrap Tooltip Body integration
 * @see [Bootstrap Tooltip]{@link https://getbootstrap.com/docs/4.0/components/tooltips/}
 *
 * @class TooltipBody
 * @extends React.Component
 * @author Jocelyn Faihy <jocelyn@faihy.fr>
 *
 * @root Theme.Tag
 * @property {Object} [props] - Component properties
 */
export class TooltipBody extends Component {
    static propTypes = {
        ...Tag.propTypes,
    };

    static defaultProps = {
        tag: 'section'
    };

    render() {
        const { className, ...props } = this.props;
        const classes = Classnames(className, 'tooltip-body');
        return <Tag {...props} className={classes} />
    }    
}